//
//      **************************************************************
//       JdeBP C++ Library Routines      General Public Licence v1.00
//          Copyright (c) 1991,1992  Jonathan de Boyne Pollard
//      **************************************************************
//
// STREAMS I/O MANIPULATOR TEMPLATES AND FUNCTIONS
//

#if !defined(___STDDEF_H_INCLUDED)
#include <_stddef.h>
#endif
#if !defined(__IOSTREAM_H_INCLUDED)
#include <iostream.h>
#endif

#ifndef __IOMANIP_H_INCLUDED

template <class T>
class smanip {
		ios & (*fn)(ios &, T);
		T d;
public:
		smanip(ios & (*f)(ios &, T), T a) : fn(f), d(a) { }

		friend istream & operator>>(istream & s, smanip & f)
			{
				(*f.fn)(s, f.d) ;
				return s ;
			}
		friend ostream & operator<<(ostream & s, smanip & f)
			{
				(*f.fn)(s, f.d) ;
				return s ;
			}
} ;

smanip<int> setw(int) ;				// set the field width
smanip<int> setbase(int);	// set the conversion base (0, 8, 10, or 16)
smanip<int> setfill(int);			// set the fill character
smanip<int> setprecision(int);		// set the floating-point precision
smanip<long> resetiosflags(long);	// clear flag bits
smanip<long> setiosflags(long);		// set flag bits

template <class T>
class sapp {
		ios & (*fn)(ios &, T);
public:
		sapp (ios & (*f)(ios &, T)) : fn(f) { }
		smanip<T> operator()(T z) { return smanip<T>(fn, z) ; }
} ;

template <class T>
class imanip {
		istream & (*fn)(istream &, T);
		T d;
public:
		imanip(istream & (*f)(istream &, T), T z ) : fn(f), d(z) { }
		friend istream & operator>>(istream & _s, imanip & f)
			{ return(*f.fn)(_s, f.d); }
} ;

template <class T>
class iapp {
		istream & (*fn)(istream &, T);
public:
		iapp(istream & (*f)(istream &, T)) : fn(f) { }
		imanip operator()(T z) { return imanip(fn, z); }
} ;

template <class T>
class omanip {
		ostream & (*fn)(ostream &, T);
		T d;
public:
		omanip(ostream & (*f)(ostream &, T), T z ) : fn(f), d(z) { }
		friend ostream & operator<<(ostream & _s, omanip & f)
			{ return(*f.fn)(_s, f.d); }
} ;

template <class T>
class oapp {
		ostream & (*fn)(ostream &, T);
public:
		oapp(ostream & (*f)(ostream &, T)) : fn(f) { }
		omanip operator()(T z) {
				return omanip(fn, z); }
} ;

template <class T>
class Iomanip {
		iostream & (*fn)(iostream &, T);
		T d;
public:
		iomanip(iostream & (*f)(iostream &, T), T z ) :
				fn(f), d(z) { }
		friend istream & operator>>(iostream & _s, iomanip & f) {
				return(*f.fn)(_s, f.d); }
		friend ostream & operator<<(iostream & _s, iomanip & f) {
				return(*f.fn)(_s, f.d); }
} ;

template <class T>
class ioapp {
		iostream & (*fn)(iostream &, T);
public:
		ioapp(iostream & (*f)(iostream &, T)) : fn(f) { }
		iomanip operator()(T z) { return iomanip(fn, z); }
} ;

#define __IOMANIP_H_INCLUDED
#endif
