//
//		**************************************************************
//		 JdeBP C++ Library Routines 	 General Public Licence v1.00
//			Copyright (c) 1991,1992	 Jonathan de Boyne Pollard
//		**************************************************************
//
// REGULAR EXPRESSIONS
//

#ifndef _REGEXP_DEFINED

class regexp {

	enum { NSUBEXP = 10 } ;

	char *program ;		// Actual compiled regular expression

	char start ;		// Character that must begin a match or NUL
	char anchor ;		// Flag for anchoring match at beginning of line
	const char *must ;	// String that a match must include, or NULL
	int mlen ;			// Length of must string

	char *	compile	(const char *, const char **, const char = 0) ;
	size_t	compare (const char *) ;
	void  	resetsubexp (void) ;

	//
	// Stuff for compare (...)
	//
	unsigned const char *input;	// String-input pointer.
	unsigned const char *bol;	// Beginning of input, for ^ check.

	int		trymatch(const char *);
	int		match(const char *);
	int		repeat(const char *);

	//
	// Stuff for compile (...)
	//
	const unsigned char	*parse;	// Input-scan pointer.
	int 			npar;		// () count.
	unsigned char	*code;		// Code-emit pointer
	long			size;		// Code size.
	unsigned char	chset[CHAR_BIT - 3];	// Char bits for "[]" ranges
	unsigned char	stopat;		// Character to stop at

	char	*reg1	(int paren, int *flagp);
	char	*branch1(int *flagp);
	char	*piece1	(int *flagp);
	char	*atom1	(int *flagp);

	char	*reg2	(int paren, int *flagp);
	char	*branch2(int *flagp);
	char	*piece2	(int *flagp);
	char	*atom2	(int *flagp);

	char	*node	(char op);
	void	emit	(char b);
	void	insert	(char op, char *opnd);
	void	tail	(char *p, char *val);
	void	optail	(char *p, char *val);

	const char *next(const char *p);

public:
	ptrdiff_t	sstart	[NSUBEXP];		// Start of substitution
	size_t		slen	[NSUBEXP];		// Length of substitution

	regexp ( void ) ;

	regexp	(const char *source, const char **endp, const char endch = 0)
		{
			program = compile (source, endp, endch) ;
		}

	~regexp	( void ) ;

	int operator! ( void ) { return !program ; }

	const char * operator= ( const char *source )
		{
			const char *dummy ;
			program = compile (source, &dummy) ;
			return source ;
		}

	int operator== ( const char *dest ) { return compare(dest) ; }

	virtual void	error ( const char * ) ;

	size_t	subst	(const char *, const char *, char *, size_t);

} ;


#define _REGEXP_DEFINED
#endif

