//
//      **************************************************************
//       JdeBP C++ Library Routines      General Public Licence v1.00
//          Copyright (c) 1991,1992  Jonathan de Boyne Pollard
//      **************************************************************
//
//  STANDARD INPUT/OUTPUT LIBRARY (ANSI)
//

#if !defined(___STDDEF_H_INCLUDED)
#include <_stddef.h>
#endif

#if !defined(__STDIO_H_INCLUDED)

#define NULL    _NULL

// Minimum #guaranteed# simultaneously open files
#define FOPEN_MAX       15

#define _IOFBF          0x00        /* Full Buffering               */
#define _IOLBF          0x40        /* Line buffering               */
#define _IONBF          0x04        /* No buffering                 */
#define BUFSIZ          512         /* setbuf() buffer size         */
#define EOF             _EOF        /* End Of File                  */
#define FILENAME_MAX    63          /* Max filename length          */
#define L_tmpnam        63          /* Max len of tmpnam()          */
#define SEEK_CUR        1
#define SEEK_END        2
#define SEEK_SET        0
#define TMP_MAX         0x7FFF      /* Maximum no of names for tmpnam() */

#if _POSIX1_SOURCE > 0
#define L_ctermid       5           /* 'C' 'O' 'N' ':' '\000' */
#endif

#if _SYSV_SOURCE > 0
#define _NFILE          20
#define P_tmpnam        "\\"
#define SYS_OPEN        20
#endif

#ifndef _FPOS_T_DEFINED
typedef _Fpos_t fpos_t;
#define _FPOS_T_DEFINED
#endif

#ifndef _SIZE_T_DEFINED
typedef _Size_t size_t;
#define _SIZE_T_DEFINED
#endif

#ifndef _FILE_DEFINED
class FILE {
	FILE(const FILE &);             // Prevent initialisation by another
	FILE & operator=(const FILE &); // Prevent assignment
public:
	enum {
		_ISVALID =	0x0001,		// File was validly contructed
		_MYBUF =	0x0010,		// Has an internal buffer
		_HEAPBUF =	0x0020,		// Buffer was made by malloc()
		_FEOF =		0x0100,		// End-Of-File
		_FERR =		0x0200,		// Error-on-File
		_LBUF =		0x0400,		// Line buffering
		_NBUF =		0x0800,		// No buffering
		_APPEND =	0x1000,		// Append each write
		_BINARY =	0x2000,		// Binary mode file
		_READ =		0x4000,		// Read after last seek
		_WRITTEN =	0x8000,		// Write after last seek
	} ;

	unsigned short  OpenFlags;      // OS/2 and DOS open flags
	unsigned short  OpenMode;       // OS/2 and DOS open mode
	unsigned short  Attribute;      // OS/2 and DOS file attribute
	unsigned short  HandType;		// OS/2 and DOS handle type

	unsigned short  flags;          // Status flags, see below
	int             fd;             // OS file number (may be table index)
	unsigned char * buf;            // Start of buffer.
	unsigned char * bufend;         // End of allocated buffer
	unsigned char * next;           // Next char to read or write. NEVER NULL.
	unsigned char * rdend;          // End of readables. NEVER NULL.
	unsigned char * rdendsave;      // Save for above when ungetc() called
	unsigned char * wrend;          // End of writables. NEVER NULL.
	unsigned char   pushback[2];    // Pushed back characters
	unsigned char   charbuf;        // Buffer if none allocated
	unsigned char   nback;          // Number of pushed back characters
	unsigned char * tmpnam;         // Pointer to temporary name

	// For Standard Library use only :-

	// Associate with and Dissociate from O/S files and change modes
	unsigned short  __cdecl  _fmode  (const char *) ;
	unsigned short  __cdecl  _fopen  (const char *) ;
	void			__cdecl  _fdopen (int) ;
	int             __cdecl  _fclose (void) ;

	void __cdecl _MakeBuf(size_t) ;          // Free up buffer allocation

	int  __cdecl _fsetpos(fpos_t *, long, int) ;
	long __cdecl _fgetpos(fpos_t *) ;

	int __cdecl  _frprep (void) ;
	int __cdecl  _fwprep (void) ;

	FILE	( const int, const char * ) ;
	FILE	( const char *, const char * ) ;
	FILE	( int, unsigned short, unsigned short) ;
	FILE	( int, unsigned short, unsigned short, size_t) ;
	~FILE() ;
	void *  operator new (size_t) ;
	void    operator delete (void *, size_t) ;

	operator int (void) { return flags & _ISVALID ; }

	int __cdecl	_fflush(void) ;
	int __cdecl	_fgetc(void) ;
	int __cdecl	_fputc(int) ;
	int __cdecl	_getc(void)
		{
			return next < rdend ? (int)(*next++) : _fgetc() ;
		}
	int __cdecl	_putc(int c)
		{
			return next < wrend ?
					(int)(*next++ = (unsigned char)c ) : _fputc(c) ;
		}
};
#define _FILE_DEFINED
#endif

extern "C" {

void    __cdecl  clearerr    (FILE *);
int     __cdecl  fclose      (FILE *);
int     __cdecl  feof        (FILE *);
int     __cdecl  ferror      (FILE *);
int     __cdecl  fflush      (FILE *);
int     __cdecl  fgetc       (FILE *);
int     __cdecl  fgetpos     (FILE *, fpos_t *);
char *  __cdecl  fgets       (char *, int, FILE *);
FILE *  __cdecl  fopen       (const char *, const char *);
int     __cdecl  fprintf     (FILE *, const char *, ...);
int     __cdecl  fputc       (int, FILE *);
int     __cdecl  fputs       (const char *, FILE *);
int     __cdecl  fread       (void *, size_t, size_t, FILE *);
FILE *  __cdecl  freopen     (const char *, const char *, FILE *);
int     __cdecl  fscanf      (FILE *, const char *, ...);
int     __cdecl  fseek       (FILE *, long, int);
int     __cdecl  fsetpos     (FILE *, fpos_t *);
long    __cdecl  ftell       (FILE *);
size_t  __cdecl  fwrite      (const void *, size_t, size_t, FILE *);
int     __cdecl  getc        (FILE *);
int     __cdecl  getchar     (void);
char *  __cdecl  gets        (char *);
void    __cdecl  perror      (const char *);
int     __cdecl  printf      (const char *, ...);
int     __cdecl  putc        (int, FILE *);
int     __cdecl  putchar     (int);
int     __cdecl  puts        (const char *);
int     __cdecl  remove      (const char *);
int     __cdecl  rename      (const char *, const char *);
void    __cdecl  rewind      (FILE *);
int     __cdecl  scanf       (const char *, ...);
void    __cdecl  setbuf      (FILE *, char *);
int     __cdecl  setvbuf     (FILE *, char *, int, size_t);
int     __cdecl  sprintf     (char *, const char *, ...);
int     __cdecl  sscanf      (const char *, const char *, ...);
FILE *  __cdecl  tmpfile     (void);
char *  __cdecl  tmpnam      (char *);
int     __cdecl  ungetc      (int, FILE *);
int     __cdecl  vfprintf    (FILE *, const char *, char *);
int     __cdecl  vprintf     (const char *, char *);
int     __cdecl  vsprintf    (char *, const char *, char *);

#if _POSIX1_SOURCE > 0
FILE *  __cdecl  fdopen      (int, const char *);
int     __cdecl  fileno      (FILE *);
#endif

#if _POSIX2_SOURCE > 0
FILE *  __cdecl  popen       (const char *, const char *);
int     __cdecl  pclose      (FILE *);
#endif

#if _MSDOS_SOURCE > 0
int     __cdecl  fgetchar    (void);
int     __cdecl  fputchar    (int);
int     __cdecl  getw        (FILE *);
int     __cdecl  putw        (FILE *);
int     __cdecl  flushall    (void);
char *  __cdecl  tempnam     (char *, char *);
#endif

extern FILE **	_Files ;		// Array of opened files
extern int		_NFiles ;		// Size of _Files currently allocated

};

#define _IOFEOF         FILE::_FEOF
#define _IOFERR         FILE::_FERR
#define _IOFLBUF        FILE::_LBUF
#define _IOFNBUF        FILE::_NBUF

// Do NOT rely on these being lvalues
#define stdin   ((FILE *const)(_Files[0]))
#define stdout  ((FILE *const)(_Files[1]))
#define stderr  ((FILE *const)(_Files[2]))

#if _MSDOS_SOURCE > 0
#define stdaux  ((FILE *const)(_Files[3]))
#define stdprn  ((FILE *const)(_Files[4]))
#endif

// Only getc() and putc() may have side effects
#define feof(s)     ((s)->flags & FILE::_FEOF)
#define ferror(s)   ((s)->flags & FILE::_FERR)
#define getc(s)     (s)->_getc()
#define putc(c,s)   (s)->_putc(c)
#define getchar()   fgetc(stdin)
#define putchar(c)  fputc((c),stdout)

#define fgetpos(f,p)    (int)(f->_fgetpos(p))
#define ftell(f)        f->_fgetpos(NULL)
#define fsetpos(f,p)    f->_fsetpos(p,0L,SEEK_SET)
#define fseek(f,o,w)    f->_fsetpos(NULL,o,w)

#if _POSIX1_SOURCE > 0
#define fileno(fh)  ((int)(fh)->fd)
#endif

#define __STDIO_H_INCLUDED
#endif
